pub enum LirScalarExpr {
Column(usize, TreatAsEqual<Option<Arc<str>>>),
Literal(Result<Row, EvalError>, ReprColumnType),
CallUnary {
func: UnaryFunc,
expr: Box<LirScalarExpr>,
},
CallBinary {
func: BinaryFunc,
expr1: Box<LirScalarExpr>,
expr2: Box<LirScalarExpr>,
},
CallVariadic {
func: VariadicFunc,
exprs: Vec<LirScalarExpr>,
},
If {
cond: Box<LirScalarExpr>,
then: Box<LirScalarExpr>,
els: Box<LirScalarExpr>,
},
}Expand description
Scalar expressions, as appear in MFPs. This is the stable, low-level, LIR definition of scalr expressions.
Variants
Column(usize, TreatAsEqual<Option<Arc<str>>>)
A column of the input row
Literal(Result<Row, EvalError>, ReprColumnType)
A literal value. (Stored as a row, because we can’t own a Datum)
CallUnary
A function call that takes one expression as an argument.
CallBinary
A function call that takes two expressions as arguments.
Fields
func: BinaryFuncFunction
expr1: Box<LirScalarExpr>First argument
expr2: Box<LirScalarExpr>Second argument
CallVariadic
A function call that takes an arbitrary number of arguments.
If
Conditionally evaluated expressions.
It is important that then and els only be evaluated if
cond is true or not, respectively. This is the only way
users can guard execution (other logical operator do not
short-circuit) and we need to preserve that.
Fields
cond: Box<LirScalarExpr>Condition
then: Box<LirScalarExpr>Then branch
els: Box<LirScalarExpr>Else branch
Implementations
Sourceimpl LirScalarExpr
impl LirScalarExpr
Sourcepub fn literal(res: Result<Datum<'_>, EvalError>, typ: ReprScalarType) -> Self
pub fn literal(res: Result<Datum<'_>, EvalError>, typ: ReprScalarType) -> Self
Packs a Datum or EvalError into a literal row of the given type.
Sourcepub fn literal_ok(datum: Datum<'_>, typ: ReprScalarType) -> Self
pub fn literal_ok(datum: Datum<'_>, typ: ReprScalarType) -> Self
Generates a literal of the given type.
Sourcepub fn as_literal(&self) -> Option<Result<Datum<'_>, &EvalError>>
pub fn as_literal(&self) -> Option<Result<Datum<'_>, &EvalError>>
If the expression is a literal, this returns the literal’s Datum or the literal’s EvalError. Otherwise, it returns None.
Sourcepub fn is_literal_true(&self) -> bool
pub fn is_literal_true(&self) -> bool
Returns true if the expression is a literal true.
Sourcepub fn as_literal_int64(&self) -> Option<i64>
pub fn as_literal_int64(&self) -> Option<i64>
If the expression is an int64, returns the literal.
Sourcepub fn call_unary<U: Into<UnaryFunc>>(self, func: U) -> Self
pub fn call_unary<U: Into<UnaryFunc>>(self, func: U) -> Self
Calls a unary function, with self as the argument.
Sourcepub fn call_binary<B: Into<BinaryFunc>>(self, other: Self, func: B) -> Self
pub fn call_binary<B: Into<BinaryFunc>>(self, other: Self, func: B) -> Self
Calls a binary function, with self as the first argument other as the second.
Sourcepub fn visit_pre<F>(&self, f: F)where
F: FnMut(&Self),
pub fn visit_pre<F>(&self, f: F)where
F: FnMut(&Self),
Visits all subexpressions in DFS preorder.
Sourcepub fn visit_pre_mut<F: FnMut(&mut Self)>(&mut self, f: F)
pub fn visit_pre_mut<F: FnMut(&mut Self)>(&mut self, f: F)
Iterative pre-order visitor.
Sourcepub fn children(&self) -> impl DoubleEndedIterator<Item = &Self>
pub fn children(&self) -> impl DoubleEndedIterator<Item = &Self>
Iterates through references to child expressions.
Sourcepub fn children_mut(&mut self) -> impl DoubleEndedIterator<Item = &mut Self>
pub fn children_mut(&mut self) -> impl DoubleEndedIterator<Item = &mut Self>
Iterates through mutable references to child expressions.